#!/usr/bin/env python
#-------------------------------------------------------------------------------
# Copyright 2008-2012 Istituto Nazionale di Fisica Nucleare (INFN)
# 
# Licensed under the EUPL, Version 1.1 only (the "Licence").
# You may not use this work except in compliance with the Licence. 
# You may obtain a copy of the Licence at:
# 
# http://joinup.ec.europa.eu/system/files/EN/EUPL%20v.1.1%20-%20Licence.pdf
# 
# Unless required by applicable law or agreed to in 
# writing, software distributed under the Licence is 
# distributed on an "AS IS" basis, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. 
# See the Licence for the specific language governing 
# permissions and limitations under the Licence.
#-------------------------------------------------------------------------------
"""
WNoD job starter script
"""

import socket
import commands
import os
import time
import re
import sys

TMPFILE   = "/tmp/my_bait"
NS_HOST   = "lsfnfs.cr.cnaf.infn.it"
NS_PORT   = 8211
BAIT_PORT = 8111

def whoIs_TheBait():
    try:
        for line in open(TMPFILE):
            if job_id in line:
                my_bait = line.strip().split("%")[2]
                break
        else:
            raise IOError
    except IOError:
        # jump here if file not found or if the job ID cannot be read
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((NS_HOST, NS_PORT))
        sock.sendall('PreExec:WhoIsTheBait\n')
        socketfile = sock.makefile('r', 0)
        my_bait = socketfile.readline().strip()
        sock.shutdown(2)
        sock.close()
        flog = open(TMPFILE, 'a')
        flog.write('BAIT%' + job_id + '%' + my_bait + '\n')
        flog.close()
        os.chmod(TMPFILE, 0777)
        
    return my_bait

hostname = socket.gethostname()
job_id = os.environ.get("LSB_JOBID")

if "bait" in hostname:
     # it's a bait
    user_details = commands.getoutput('id ' + os.environ.get("USER"))
    details = ''
    vm_type = 'default'
    vm_ram = 'default'
    vm_cpu = 'default'
    real_cmd = ' '.join(sys.argv[1:])
    if os.path.isfile(real_cmd):
        for line in open(real_cmd):
            if re.search('WNoD_VM', line):
                variable = line.strip().split(':')[0].split()[1]
                value = line.strip().split(':')[1]
                if variable.split('_')[2] == 'TYPE':
                    vm_type =  value
                if variable.split('_')[2] == 'RAM':
                    vm_ram =  value
                if variable.split('_')[2] == 'CPU':
                    vm_cpu =  value
    # this could be simplified
    for i in user_details.split():
        details = details + str(i.split('(')[1].split(')')[0]) + '^'

    lsfcommand = 'source /etc/profile.d/lsf.sh; bstop ' + str(job_id)
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect((hostname, BAIT_PORT))
    sock.sendall('lsfjobid:' + str(job_id) + ':' + details + vm_type +'^' + vm_ram +'^' + vm_cpu +':firstJS\n')
    sock.shutdown(2)
    sock.close()
    es = commands.getoutput(lsfcommand)
    flog = open('/tmp/orru', 'a')
    flog.write(time.ctime(time.time( )) + '-- SYS ARGV--'+str(es)+'\n')  
    flog.close()

elif hostname.startswith("vwn-"):
    # it's a virtual worker node
    
    # call job accounting script
    ACCOUNTING_SCRIPT = "/usr/share/lsf/dyndb.sh"
    output = commands.getoutput(ACCOUNTING_SCRIPT)
    
    bait = whoIs_TheBait() 
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect((bait.split(':')[0], BAIT_PORT))
    sock.sendall('job_id:' + str(job_id) +':secondJS' + '\n')
    sock.shutdown(2)
    sock.close()
    string = ' '.join(sys.argv[:])
    real_cmd = ' '.join(sys.argv[1:])
    status,output = commands.getstatusoutput('/usr/share/lsf/conf/scripts/jobstarter-logbook.sh')

elif hostname.startswith("wn-"):
    # it's a physical worker node
    status,output = commands.getstatusoutput('/usr/share/lsf/conf/scripts/jobstarter-logbook.sh')

elif hostname.startswith("sb"):
    # it's a software build host
    status,output = commands.getstatusoutput('/usr/share/lsf/conf/scripts/jobstarter-logbook.sh')
